@@ -28,14 +28,15 @@ int touch_directory(Options *op, const char *victim);
 int copy_file(Options *op, const char *srcfile,
               const char *dstfile, mode_t mode);
 char *write_temp_file(Options *op, const int len,
-                      const unsigned char *data, mode_t perm);
+                      const void *data, mode_t perm);
 int set_destinations(Options *op, Package *p); /* XXX move? */
 int get_prefixes(Options *op); /* XXX move? */
 void add_kernel_modules_to_package(Options *op, Package *p);
-void remove_non_kernel_module_files_from_package(Options *op, Package *p);
-void remove_opengl_files_from_package(Options *op, Package *p);
-void remove_wine_files_from_package(Options *op, Package *p);
-void remove_systemd_files_from_package(Options *op, Package *p);
+void remove_non_kernel_module_files_from_package(Package *p);
+void remove_non_installed_kernel_module_source_files_from_package(Package *p);
+void remove_opengl_files_from_package(Package *p);
+void remove_wine_files_from_package(Package *p);
+void remove_systemd_files_from_package(Package *p);
 int mode_string_to_mode(Options *op, char *s, mode_t *mode);
 char *mode_to_permission_string(mode_t mode);
 int confirm_path(Options *op, const char *path);
@@ -63,7 +64,7 @@ void process_dkms_conf(Options *op, Package *p);
 int set_security_context(Options *op, const char *filename, const char *type);
 void get_default_prefixes_and_paths(Options *op);
 void get_compat32_path(Options *op);
-char *nv_strreplace(char *src, char *orig, char *replace);
+char *nv_strreplace(const char *src, const char *orig, const char *replace);
 char *get_filename(Options *op, const char *def, const char *msg);
 int secure_delete(Options *op, const char *file);
 void invalidate_package_entry(PackageEntry *entry);
