@@ -51,8 +51,9 @@
 /* local prototypes */
 
 
+static Package *parse_manifest(Options *op);
 static int install_kernel_modules(Options *op,  Package *p);
-void free_package(Package *p);
+static void free_package(Package *p);
 static int assisted_module_signing(Options *op, Package *p);
 
 static const KernelModuleInfo optional_modules[] = {
@@ -722,7 +723,7 @@ static int parse_kernel_modules_list(Package *p, char *list) {
  *   - symbolic links will name the target of the link
  */
 
-Package *parse_manifest (Options *op)
+static Package *parse_manifest (Options *op)
 {
     char *buf, *c, *tmpstr;
     int line;
@@ -804,9 +805,31 @@ Package *parse_manifest (Options *op)
 
     if (op->kernel_module_build_directory_override) {
         p->kernel_module_build_directory =
-            nvstrdup(op->kernel_module_build_directory_override);
+            op->kernel_module_build_directory_override;
+        op->kernel_module_build_directory_override = NULL;
     } else {
-        p->kernel_module_build_directory = nvstrdup("kernel");
+        struct module_type_info types;
+        int num_types;
+
+        num_types = valid_kernel_module_types(op, &types, FALSE);
+
+        if (num_types > 0) {
+            int selection;
+
+            if (num_types == 1) {
+                selection = 0;
+            } else {
+                selection = ui_multiple_choice(op, types.licenses, num_types,
+                    types.default_entry,
+                    "Multiple kernel module types are available for this "
+                    "system. Which would you like to use?"
+                );
+            }
+
+            p->kernel_module_build_directory = nvstrdup(types.dirs[selection]);
+        } else {
+            p->kernel_module_build_directory = nvstrdup("kernel");
+        }
     }
 
     remove_trailing_slashes(p->kernel_module_build_directory);
@@ -871,6 +894,9 @@ Package *parse_manifest (Options *op)
             case FILE_TYPE_VULKAN_ICD_JSON:
                 op->vulkan_icd_json_packaged = TRUE;
                 break;
+            case FILE_TYPE_VULKANSC_ICD_JSON:
+                op->vulkansc_icd_json_packaged = TRUE;
+                break;
             default: break;
         }
 
@@ -1075,7 +1101,7 @@ void add_package_entry(Package *p,
  * free_package() - free the Package data structure
  */
 
-void free_package(Package *p)
+static void free_package(Package *p)
 {
     int i;
 
